<?php
use  PrestaShop\PrestaShop\Core\Payment\PaymentOption;
if (!defined('_PS_VERSION_')) {
    exit;
}


require_once _PS_MODULE_DIR_ . 'Fiserv/classes/FiservApiHandler.php';
require_once _PS_MODULE_DIR_ . 'Fiserv/classes/FiservFileLogger.php';
require_once _PS_MODULE_DIR_ . 'Fiserv/classes/FiservTools.php';
require_once _PS_MODULE_DIR_ . 'Fiserv/classes/FiservUtility.php';
require_once _PS_MODULE_DIR_ . 'Fiserv/classes/FiservRequest.php';
require_once _PS_MODULE_DIR_ . 'Fiserv/classes/FiservResponse.php';

/**
 * Fiserv payment module main class.
 */
class Fiserv extends PaymentModule {

    private $logger;

    /**
     * Constructor.
     */
    public function __construct() {
        $this->name = 'Fiserv';
        $this->tab = 'payments_gateways';
        $this->version = '2.1.1';
        $this->author = 'Fiserv';
        $this->controllers = array('payment','failure','success','notify');
        $this->is_eu_compatible = 1;

        $this->logger = FiservTools::getLogger();

        // check version compatibility
        $minor = Tools::substr(_PS_VERSION_, strrpos(_PS_VERSION_, '.') + 1);
        $replace = (int) $minor + 1;
        $start = Tools::strlen(_PS_VERSION_) - Tools::strlen($minor);
        $version = substr_replace(_PS_VERSION_, (string) $replace, $start);
        $this->ps_versions_compliancy = array('min' => '1.7.8.8', 'max' => $version);

        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        parent::__construct();

        $order_id = (int) Tools::getValue('id_order', 0);


        $this->order = new Order($order_id);
        
        if (($this->order->module == $this->name) && ($this->context->controller instanceof OrderConfirmationController)) {
            // patch to use different display name according to the used payment sub-module
            $this->displayName = $order->payment;
        } else {
            $this->displayName = Configuration::get('PLUGIN_NAME');
            if ($this->displayName == '')
                $this->displayName = 'Fiserv';
        }

        $this->description = $this->l('Accept payments by credit cards');
        $this->confirmUninstall = $this->l('Are you sure you want to delete your module details ?');

        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }
    }

    /**
     * @see PaymentModuleCore::install()
     */
    public function install() {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            // incompatible version of PrestaShop
            return false;
        }

        if (version_compare(_PS_VERSION_, '1.6', '>')) {
            if (!parent::install() || !$this->registerHook('header') || !$this->registerHook('paymentReturn')
                    || !$this->registerHook('adminOrder') || !$this->registerHook('actionOrderSlipAdd') || !$this->registerHook('actionProductDelete') || !$this->registerHook('actionProductCancel')|| !$this->registerHook('actionOrderPostProcess') ) {
                return false;
            }
        } 
     
        if (version_compare(_PS_VERSION_, '1.7', '<')) {
            if (!$this->registerHook('payment') || !$this->registerHook('displayPaymentEU') || !$this->registerHook('adminOrder') || !$this->registerHook('displayHeader')) {
                return false;
            }

        } else {
            if (!$this->registerHook('paymentOptions')) {
                return false;
            }
        }
       
        // set default values
        foreach (FiservTools::getAdminParameters() as $param) {
            if (in_array($param['key'], FiservTools::$multi_lang_fields)) {
                $default = FiservTools::convertIsoArrayToIdArray($param['default']);
            } elseif (is_array($param['default'])) {
                $default = serialize($param['default']);
            } else {
                $default = $param['default'];
            }

            if (!Configuration::updateValue($param['key'], $default, false, false, false)) {
                return false;
            }
        }

        if (!Db::getInstance()->Execute('CREATE TABLE IF NOT EXISTS ps_saved_cards (
		  saved_card_id int NOT NULL AUTO_INCREMENT,
		  customer_id int(10) unsigned NOT NULL,
		  token_id varchar(200) NOT NULL, 
		  pseudo_cc_no varchar(25) NOT NULL, 
		  brand varchar(25) NOT NULL, 
		  record_date timestamp default now(),
		  PRIMARY KEY (saved_card_id)
		) ENGINE=' . _MYSQL_ENGINE_ . '  DEFAULT CHARSET=utf8'))
            return false;

        if (!Db::getInstance()->Execute('CREATE TABLE IF NOT EXISTS ps_firstdata_order (
		  fd_order_id int(10) NOT NULL AUTO_INCREMENT,
          id_cart int(10) unsigned NOT NULL DEFAULT 0,
		  id_transaction varchar(50) NOT NULL,
		  payment_status varchar(20) NOT NULL,
		  payment_type varchar(20) NOT NULL,
		  currency_code varchar(20) NOT NULL,
		  order_date timestamp default now(),
		  payment_response text,
		  PRIMARY KEY (fd_order_id)
		) ENGINE=' . _MYSQL_ENGINE_ . '  DEFAULT CHARSET=utf8'))
            return false;

        if(!Db::getInstance()->Execute('UPDATE ' . _DB_PREFIX . 'configuration SET value = `PosNet` WHERE value = `ePosNet`')) return false;
        if(!Db::getInstance()->Execute('ALTER TABLE ps_firstdata_order RENAME COLUMN id_order TO id_cart;')) return false;

        // clear module compiled templates
        $tpls = array(
            'bc/payment',
        );
        foreach ($tpls as $tpl) {
            $this->context->smarty->clearCompiledTemplate($this->getTemplatePath($tpl . '.tpl'));
        }



       

        return true;
    }


    /**
     * @see PaymentModuleCore::uninstall()
     */
    public function uninstall() {
        $result = true;
        foreach (FiservTools::getAdminParameters() as $param) {
            $result &= Configuration::deleteByName($param['key']);
        }

        // delete all obsolete Fiserv params but not custom order states
        $result &= Db::getInstance()->execute(
                'DELETE FROM `' . _DB_PREFIX_ . "configuration` WHERE `name` LIKE 'IPG_%' AND `name` NOT LIKE 'IPG_OS_%'"
        );

        return $result && parent::uninstall();
    }
    
    public function hookdisplayHeader()
    {
      if (!$this->active)
        return;
    
        $this->context->controller->addJs($this->getPathUri() . 'views/js/FiservPayments.js');
        $this->context->controller->addCSS($this->getPathUri() . 'views/css/Fiserv_bc.css');
  
    }
    /**
     * Admin form management.
     * @return string
     */
    public function getContent() {
        $msg = '';

        if (Tools::isSubmit('Fiserv_submit_admin_form')) {
            $this->postProcess();

            if (empty($this->_errors)) {
                // no error, display update ok message
                $msg .= $this->displayConfirmation($this->l('Settings updated.'));
            } else {
                // display errors
                $msg .= $this->displayError(implode('<br />', $this->_errors));
            }

            $msg .= '<br />';
        }

        return $msg . $this->renderForm();
    }

    /**
     * Validate and save module admin parameters.
     */
    private function postProcess() {

        require_once _PS_MODULE_DIR_ . 'Fiserv/classes/FiservRequest.php';
        $request = new FiservRequest(); // new instance of FiservRequest for parameters validation

      
        $fileExt = false;
        // load and validate from request

  
        foreach (FiservTools::getAdminParameters() as $param) {
            $key = $param['key']; // PrestaShop parameter key
            // if (!Tools::getIsset($key)) {
            // // if field is disabled, don't save it
            // continue;
            // }

            $label = $this->l($param['label'], 'back_office'); // translated human-readable label
            $name = isset($param['name']) ? $param['name'] : null; // Fiserv API parameter name

            $value = Tools::getValue($key, null);

            $IPG_API_CERTIFICATE_PEM = Configuration::get('IPG_API_CERTIFICATE_PEM');
            $IPG_API_CERTIFICATE_KEY = Configuration::get('IPG_API_CERTIFICATE_KEY');
            $IPG_API_SERVER_TRUST_PEM = Configuration::get('IPG_API_SERVER_TRUST_PEM');
            $upload_dir = _PS_UPLOAD_DIR_;
            $upload_dir = str_replace('\\', '/', $upload_dir);

            if ($value === '') { // consider empty strings as null
                $value = null;
            }

            if (in_array($key, FiservTools::$multi_lang_fields)) {
                if (!is_array($value) || empty($value)) {
                    $value = array();
                }
            } elseif (in_array($key, FiservTools::$group_amount_fields)) {
                if (!is_array($value) || empty($value)) {
                    $value = array();
                } else {
                    $error = false;
                    foreach ($value as $id => $option) {
                        if ($option['min_amount'] && !is_numeric($option['min_amount']) || $option['min_amount'] < 0) {
                            $value[$id]['min_amount'] = ''; // error, reset incorrect value
                            $error = true;
                        }

                        if ($option['max_amount'] && !is_numeric($option['max_amount']) || $option['max_amount'] < 0) {
                            $value[$id]['max_amount'] = ''; // error, reset incorrect value
                            $error = true;
                        }
                    }

                    if ($error) {
                        $this->_errors[] = sprintf($this->l('One or more values are invalid for field « %s ». Only valid entries are saved.'), $label);
                    }
                }

                $value = serialize($value);
            } elseif ($key === 'INS_MULTI_OPTIONS') {

                if (!is_array($value) || empty($value)) {
                    $value = array();
                } else {

                    $error = false;
                    $error1 = false;
                    $error2 = false;

                    foreach ($value as $id => $option) {
                        if (empty($option['count']) || empty($option['label']) || empty($option['min_amount']) || empty($option['max_amount'])) {
                            // error, do not save this option
                            $error = true;
                            unset($value[$id]);
                            //Min amount, max amount ,count, days and 1st payment fields having values
                        } else if (!is_numeric($option['count']) || !is_numeric($option['min_amount']) || !is_numeric($option['max_amount'])) {
                            unset($value[$id]);
                            $error1 = true;
                        } else if ($option['max_amount'] <= $option['min_amount']) {
                            $error2 = true;
                        } else{
                            $default = is_string($option['label']) && $option['label'] ?
                                    $option['label'] : $option['count'] . ' x';
                            $option_label = is_array($option['label']) ? $option['label'] : array();

                            foreach (Language::getLanguages(false) as $language) {
                                $lang = $language['id_lang'];
                                if (!isset($option_label[$lang]) || empty($option_label[$lang])) {
                                    $option_label[$lang] = $default;
                                }
                            }

                            $value[$id]['label'] = $option_label;
                            if (isset($option['installInt'])){
                                $value[$id]['installInt'] = 'yes';
                            }else{
                                $value[$id]['installInt'] = 'no';
                            }

                        }
                    }
                    if ($error) {
                        $this->_errors[] = sprintf($this->l('Min amount, max amount, count and period are mandatory'), $label);
                    }
                    if ($error1) {
                        $this->_errors[] = sprintf($this->l('Min amount, max amount, count and period should be numberic'), $label);
                    }

                    if ($error2) {
                        $this->_errors[] = sprintf($this->l('Max amount is greater than min amount'), $label);
                    }
                }

             
                $value = serialize($value);

              
            } elseif (in_array($key, FiservTools::$amount_fields)) {
                if (!empty($value) && (!is_numeric($value) || $value < 0)) {
                    $this->_errors[] = sprintf($this->l('Invalid value «%1$s» for field «%2$s».'), $value, $label);
                    continue;
                }
            } elseif ($key === 'PAYMENT_METHOD') {               
                $value = serialize($value);
            } elseif ($key === 'IPG_API_SERVER_TRUST_PEM') {
                $fname_server = $_FILES['IPG_API_SERVER_TRUST_PEM']['name'];
                $fname_server_type = $_FILES['IPG_API_SERVER_TRUST_PEM']['type'];
                if ($fname_server != '') {
                    $value = $fname_server;
                } else {
                    $value = $IPG_API_SERVER_TRUST_PEM;
                }
                $field = 'IPG_API_SERVER_TRUST_PEM';
            } elseif ($key === 'IPG_API_CERTIFICATE_PEM') {
                $fname_certificate = $_FILES['IPG_API_CERTIFICATE_PEM']['name'];
                $fname_certificate_type = $_FILES['IPG_API_CERTIFICATE_PEM']['type'];
                if ($fname_certificate != '') {
                    $value = $fname_certificate;
                } else {
                    $value = $IPG_API_CERTIFICATE_PEM;
                }
                $field = 'IPG_API_CERTIFICATE_PEM';
            } elseif ($key === 'IPG_API_CERTIFICATE_KEY') {
                $fname_certificate_key = $_FILES['IPG_API_CERTIFICATE_KEY']['name'];
                $fname_certificate_key_type = $_FILES['IPG_API_CERTIFICATE_KEY']['type'];
                if ($fname_certificate_key != '') {
                    $value = $fname_certificate_key;
                } else {
                    $value = $IPG_API_CERTIFICATE_KEY;
                }
                $field = 'IPG_API_CERTIFICATE_KEY';
            } elseif ($key === 'IPG_API_CERTIFICATE_PASSWORD') {
                $value = $_POST['IPG_API_CERTIFICATE_PASSWORD'];
            }elseif ($key === 'PLUGIN_LOGO') {
                $value = $_POST['PLUGIN_LOGO'];
            }


            if ($key === 'IPG_API_CERTIFICATE_KEY' || $key === 'IPG_API_CERTIFICATE_PEM' || $key === 'IPG_API_SERVER_TRUST_PEM') {
                if ($field != '') {
                    $uploads_dir = '../upload';
                    if (!is_dir($uploads_dir)) {
                        mkdir($uploads_dir);
                        chmod($uploads_dir, 0777);
                    }
                    $tmp_name = $_FILES["$field"]["tmp_name"];
                    $names = basename($_FILES["$field"]["name"]);
                    move_uploaded_file($tmp_name, "$uploads_dir/$names");
                }
            }

            // validate with FiservRequest
            if ($name) {
                $values = is_array($value) ? $value : array($value); // to check multilingual fields
                $error = false;

                foreach ($values as $v) {
                    if (!$request->set($name, $v)) {
                        $error = true;
                        if (empty($v)) {
                            $this->_errors[] = sprintf($this->l('The field «%s» is mandatory.'), $label);
                        } else {
                            $this->_errors[] = sprintf($this->l('Invalid value «%1$s» for field «%2$s».'), $v, $label);
                        }
                    }
                }

                if ($error) {
                    continue; // not save fields with errors
                }
            }            
           
            // valid field : try to save into DB
            if (!Configuration::updateValue($key, $value)) {
                $this->_errors[] = sprintf($this->l('Problem occured while saving field «%s».'), $label);
            } else {
                // temporary variable set to update PrestaShop cache
                Configuration::set($key, $value);
            }           
        }      
    }

    private function renderForm() {
        $this->addJS('Fiserv.js');
        $this->context->controller->addJqueryUI('ui.accordion');

        $html = '';

        if (version_compare(_PS_VERSION_, '1.6', '>=')) {
            $html .= '<style type="text/css">
                            #content {
                                min-width: inherit !important;
                            }
                     </style>';
            $html .= "\n";
        }

        require_once _PS_MODULE_DIR_ . 'Fiserv/classes/admin/FiservHelperForm.php';
       
        $this->context->smarty->assign(FiservHelperForm::getAdminFormContext());
        $this->context->smarty->assign('features', FiservUtility::getFeatures());

        $country = Configuration::get('COUNTRY');


        $this->context->smarty->assign('country', $country);
        $reseller = Configuration::get('RESELLER');
        $this->context->smarty->assign('reseller', $reseller);
        $loc_pay_method = @unserialize(Configuration::get('PAYMENT_METHOD'));
        $loc_pay_method_list = '';
        $local_payment = Configuration::get('LOCAL_PAYMENT');
        
        if(!empty($loc_pay_method) && ($local_payment == 'yes')){
            $loc_pay_method_list = implode(',', $loc_pay_method);
        }


        $this->context->smarty->assign('loc_pay_method', $loc_pay_method_list);
       
        $this->context->smarty->assign('local_payment', $local_payment);
        $ins_enb = Configuration::get('INSTALMENT_ENABLED');
        $this->context->smarty->assign('ins_enb', $ins_enb);
        $dcc_enb = Configuration::get('DCC_SKIP_OFFER');
        $this->context->smarty->assign('dcc_enb', $dcc_enb);
        $secure_pay_enb = Configuration::get('SECURE_PAY');
        $this->context->smarty->assign('secure_pay_enb', $secure_pay_enb);

    	 $logs_dir = _PS_ROOT_DIR_.DIRECTORY_SEPARATOR.'var'.DIRECTORY_SEPARATOR.'logs'.DIRECTORY_SEPARATOR;
       // $logs_dir = '/prestashop/var/logs/';
        $logs_dir_path = $logs_dir . 'Fiserv.log';
        $this->context->smarty->assign('logs_dir_path', $logs_dir_path);

        $upload_dir = _PS_UPLOAD_DIR_;
        $upload_dir = str_replace('\\', '/', $upload_dir);
        $IPG_API_CERTIFICATE_KEY = Configuration::get('IPG_API_CERTIFICATE_KEY');
        if ($IPG_API_CERTIFICATE_KEY != '')
            $IPG_API_CERTIFICATE_KEY = $upload_dir . $IPG_API_CERTIFICATE_KEY;
        $this->context->smarty->assign('GET_IPG_API_CERTIFICATE_KEY', $IPG_API_CERTIFICATE_KEY);

        $IPG_API_SERVER_TRUST_PEM = Configuration::get('IPG_API_SERVER_TRUST_PEM');
        if ($IPG_API_SERVER_TRUST_PEM != '')
            $IPG_API_SERVER_TRUST_PEM = $upload_dir . $IPG_API_SERVER_TRUST_PEM;
        $this->context->smarty->assign('GET_IPG_API_SERVER_TRUST_PEM', $IPG_API_SERVER_TRUST_PEM);

        $IPG_API_CERTIFICATE_PEM = Configuration::get('IPG_API_CERTIFICATE_PEM');
        if ($IPG_API_CERTIFICATE_PEM != '')
            $IPG_API_CERTIFICATE_PEM = $upload_dir . $IPG_API_CERTIFICATE_PEM;
        $this->context->smarty->assign('GET_IPG_API_CERTIFICATE_PEM', $IPG_API_CERTIFICATE_PEM);

        
        $this->context->smarty->assign('LABELS', json_encode(FiservUtility::getLabels()));

        $this->context->smarty->assign('PLUGIN_LOGO', Configuration::get('PLUGIN_LOGO'));

        
        $form = $this->context->smarty->fetch(_PS_MODULE_DIR_ . 'Fiserv/views/templates/admin/back_office.tpl');

        $prefered_post_vars = 0;
        $prefered_post_vars += substr_count($form, 'name="IPG_');
        $prefered_post_vars += 100; // to take account of dynamically created inputs

        if ((ini_get('suhosin.post.max_vars') && ini_get('suhosin.post.max_vars') < $prefered_post_vars)
                || (ini_get('suhosin.request.max_vars') && ini_get('suhosin.request.max_vars') < $prefered_post_vars)) {
            $html .= $this->displayError(sprintf($this->l('Warning, please increase the suhosin patch for PHP post and request limits to save module configurations correctly. Recommended value is %s.'), $prefered_post_vars));
        } elseif (ini_get('max_input_vars') && ini_get('max_input_vars') < $prefered_post_vars) {
            $html .= $this->displayError(sprintf($this->l('Warning, please increase the value of the max_input_vars directive in php.ini to to save module configurations correctly. Recommended value is %s.'), $prefered_post_vars));
        }

        $html .= $form;
        return $html;
    }

    /**
     * Payment method selection page header.
     *
     * @param array $params
     * @return string|void
     */
    public function hookHeader($params) {
        $controller = $this->context->controller;
        if ($controller instanceof OrderController || $controller instanceof OrderOpcController) {
            if (isset($this->context->cookie->FiservPayErrors)) {
                // process errors from other pages
                $controller->errors = array_merge(
                        $controller->errors, explode("\n", $this->context->cookie->FiservPayErrors)
                );
                unset($this->context->cookie->FiservPayErrors);

                // unset HTTP_REFERER from global server variable to avoid back link display in error message
                $_SERVER['HTTP_REFERER'] = null;
                $this->context->smarty->assign('server', $_SERVER);
            }

            // add main module CSS
            $this->addCss('Fiserv.css');

            // add backward compatibility module CSS
            $html = '';
            if (version_compare(_PS_VERSION_, '1.7', '<')) {
                $this->addCss('Fiserv_bc.css');

                // load payment module style to apply it to our tag
                if ($this->useMobileTheme()) {
                    $css_file = _PS_THEME_MOBILE_DIR_ . 'css/global.css';
                } else {
                    $css_file = _PS_THEME_DIR_ . 'css/global.css';
                }

                $css = Tools::file_get_contents(str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $css_file));

                $matches = array();
                $res = preg_match_all('#(p\.payment_module(?:| a| a\:hover) ?\{[^\}]+\})#i', $css, $matches);
                if ($res && !empty($matches) && isset($matches[1]) && is_array($matches[1]) && !empty($matches[1])) {
                    $html .= '<style type="text/css">' . "\n";
                    $html .= str_ireplace('p.payment_module', 'div.payment_module', implode("\n", $matches[1])) . "\n";
                    $html .= '</style>' . "\n";
                }
            }

            return $html;
        }
    }

    protected function useMobileTheme() {
        if (method_exists(get_parent_class($this), 'useMobileTheme')) {
            return parent::useMobileTheme();
        } elseif (method_exists($this->context, 'getMobileDevice')) {
            return ($this->context->getMobileDevice() && file_exists(_PS_THEME_MOBILE_DIR_ . 'layout.tpl'));
        } else {
            return false;
        }
    }

    private function addJs($js_file) {
        $controller = $this->context->controller;

        if (method_exists($controller, 'registerJavascript')) { // PrestaShop 1.7
            $controller->registerJavascript(
                    'module-Fiserv', 'modules/' . $this->name . '/views/js/' . $js_file, array('position' => 'bottom', 'priority' => 150)
            );
        } else {
            $controller->addJs($this->_path . 'views/js/' . $js_file);
        }
    }

    private function addCss($css_file) {
        $controller = $this->context->controller;

        if (method_exists($controller, 'registerStylesheet')) { // PrestaShop 1.7
            $controller->registerStylesheet(
                    'module-Fiserv', 'modules/' . $this->name . '/views/css/' . $css_file, array('media' => 'all', 'priority' => 90)
            );
        } else {
            $controller->addCss($this->_path . 'views/css/' . $css_file, 'all');
        }
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $payment_options = [
            $this->getEmbeddedPaymentOption()
        ];

        return $payment_options;
    }

    public function hookPaymentReturn()
    {
        echo "chamou o hookpaymentreturn";
        die();
    }

    public function hookAdminOrder()
    {}

    public function hookActionOrderSlipAdd($params)
    {
        $getOrderSQL = Db::getInstance()->getRow('SELECT * FROM ps_firstdata_order WHERE id_cart = "' . $params['order']->id_cart . '"');

        if(!$getOrderSQL){
            $this->logger->logInfo('Error on hookActionOrderSlipAdd: id_cart id ' . $params['order']->id_cart . ' not found on table ps_firstdata_order');
            Tools::displayError('Error on online refund. Open VT and retry refund manually');
            return false;
        }
        $method = "";        
        if ($getOrderSQL['payment_type'] == "preauth") {                       
            $method = "void";            
        } else if ($getOrderSQL['payment_type'] == "sale") {          
            $method = "return";            
        } else {             
            $message = $this->l('Partial amount refund is not possible on preauth transaction');
            return false;
        }

        $params['order_hash_id'] = $getOrderSQL['id_cart'];
        $params['transaction_id'] = $getOrderSQL['id_transaction'];
        $params['order_currency'] = $getOrderSQL['currency_code'];
        $params['method'] = $method;


        $this->init_api();
        FiservApiHandler::refund_transaction($params);
    }

    public function hookActionProductDelete()
    {}

    public function hookActionProductCancel()
    {}

    public function hookPayment($params)
    {
        if (!$this->active)
        return;
  

      return $this->generateForm();
    }

    public function getEmbeddedPaymentOption()
    {   
        global $cookie;
        $title = 'Pay with Credit Card';
        $iso_code = Language::getIsoById((int) $cookie->id_lang);
        $langArr = array('en' => 'en_EN', 'es' => 'es_ES', 'mx' => 'es_ES','br' => 'pt_BR',);
        $language = $langArr[$iso_code];
        if($language == "es_ES"){
            $title ='Paga  Con Tarjeta';
        }
  
        $embeddedOption = new PaymentOption(); 
               $embeddedOption->setCallToActionText($this->l( $title))
                                ->setForm($this->generateForm())
								->setAdditionalInformation(Configuration::get('PLUGIN_DESC'))
								->setLogo(Media::getMediaPath(Configuration::get('PLUGIN_LOGO')));

        return $embeddedOption;
    }
    
    /**
     * Payment function, display payment buttons/forms for all sub-modules.
     *
     * @param array $params
     * @return void|string
     */
    public function generateForm() {
        global $cookie;

        unset($this->context->cookie->FiservPayErrors);
        if (!$this->active) {
            return;
        }
        $iso_code = Language::getIsoById((int) $cookie->id_lang);
        $langArr = array('en' => 'en_EN', 'es' => 'es_ES', 'mx' => 'es_ES','br' => 'pt_BR',);
        $language = $langArr[$iso_code];

        $cart = $this->context->cart;

        // version tag for specific styles
        $tag = version_compare(_PS_VERSION_, '1.6', '<') ? 'ipg15' : 'ipg16';
        $this->context->smarty->assign('ipg_tag', $tag);
        $card_type=$this->getCardType();
        $logoPath = Configuration::get('PLUGIN_LOGO');
        $pluginName = Configuration::get('PLUGIN_NAME');
        $pluginDesc = Configuration::get('PLUGIN_DESC');
        $ins = Configuration::get('INSTALMENT_ENABLED');
        $authUX = Configuration::get('AUTH_METHOD');
        $logoDisplayed = Configuration::get('LOGO_DISPLAY');
        $tokenisation = Configuration::get('TOKENISATION');
        $checkout_mode = Configuration::get('CHECKOUT_MODE');
        $local_payment = Configuration::get('LOCAL_PAYMENT');


        $loc_pay_method = @unserialize(Configuration::get('PAYMENT_METHOD'));
        $loc_pay_method_list = '';
        if(!empty($loc_pay_method)){
            $loc_pay_method_list = implode(',', $loc_pay_method);
        }
        $loc_pay_methods = @unserialize(Configuration::get('PAYMENT_METHOD'));

        $this->context->smarty->assign('plugin_name', $pluginName);
        $this->context->smarty->assign('plugin_desc', '( ' . $pluginDesc . ' )');
        $this->context->smarty->assign('plu_desc', $pluginDesc);
        $this->context->smarty->assign('logo_path', $logoPath);
        $this->context->smarty->assign('logo_displayed', $logoDisplayed);
        $this->context->smarty->assign('ins', $ins);
		$this->context->smarty->assign('card_type',$card_type);
        $this->context->smarty->assign('authUX', $authUX);
        $this->context->smarty->assign('tokenisation', $tokenisation);
        $this->context->smarty->assign('checkout_mode', $checkout_mode);
        $this->context->smarty->assign('local_payment', $local_payment);
        $this->context->smarty->assign('loc_pay_method', $loc_pay_method_list);
        $this->context->smarty->assign('loc_pay_methods', $loc_pay_methods);

        $this->context->smarty->assign('language', $language);

        $local_payment = $this->paymentOptionsValues();        
        $local_payment_method_options = array();
        if(!empty($loc_pay_methods)){
            foreach ($loc_pay_methods as $local_options) {
                if (array_key_exists($local_options, $local_payment)) {
                    $local_payment_method_options[$local_options] = $local_payment[$local_options];
                }
            }
        }
        $this->context->smarty->assign('local_payment_method_options', $local_payment_method_options);


        //print_r($local_payment_method_options);die;
        if ($authUX == 'hidden') {
            if (isset($this->context->cookie->FiservPayErrors)) {
                // process errors from other pages
                $controller->errors = array_merge(
                        $controller->errors, explode("\n", $this->context->cookie->FiservPayErrors)
                );
            }
        }
        unset($this->context->cookie->FiservPayErrors);


        $dbTokenResult = array();

        if ($this->context->customer->isLogged() && $tokenisation == 'yes') {

            $dbTokenResult = Db::getInstance()->executeS('SELECT * FROM ps_saved_cards WHERE customer_id = ' . $this->context->customer->id);
        }
     
        $this->context->smarty->assign(array('storedCardTokens' => $dbTokenResult));


        
        $this->context->smarty->assign($this->getTplVars($cart));

   
        $this->context->smarty->assign('local_config', json_encode($this->paymentOptions()));


        return $this->context->smarty->fetch('module:Fiserv/views/templates/hook/bc/payment.tpl');
    }
	
    
	
	 public function getCardType() {
        $return = null;
		$country = Configuration::get('COUNTRY');
        $reseller = Configuration::get('RESELLER');   
        $feature = FiservUtility::getConfig();
         if (array_key_exists($country, $feature) && array_key_exists($reseller, $feature[$country])) {
            $return = $feature[$country][$reseller]['card_type'];
        }

        return $return;
    }

    /**
     * Init the API class and set the username/password etc.
     */
    protected function init_api() {
        $upload_dir = _PS_UPLOAD_DIR_;
        $upload_dir = str_replace('\\', '/', $upload_dir);
        FiservApiHandler::$api_url = $url = Configuration::get('IPG_API_URL');
        FiservApiHandler::$api_username = Configuration::get('IPG_API_USERNAME');
        FiservApiHandler::$api_password = Configuration::get('IPG_API_PASSWORD');
        FiservApiHandler::$certificate_key_password = Configuration::get('IPG_API_CERTIFICATE_PASSWORD');
        FiservApiHandler::$server_trust_pem = $upload_dir . Configuration::get('IPG_API_SERVER_TRUST_PEM');
        FiservApiHandler::$client_certificate_pemfile = $upload_dir . Configuration::get('IPG_API_CERTIFICATE_PEM');
        FiservApiHandler::$client_certificate_keyfile = $upload_dir . Configuration::get('IPG_API_CERTIFICATE_KEY');
    }

    /**
     * After Order postProcess called in backend.
     *
     * @param array $params
     */
   
    public function hookActionOrderPostProcess($params) {


        $this->init_api();
        $order = $params['order'];
       
        $order_id = (int) $order->id;

        $od = new Order((int) $order_id);
    
        
        $refundAmt = $params['refundAmount'];                
        $totAmount = number_format($order->total_paid, 2);        
        $cartID = (int) $order->id_cart;
        $result = Db::getInstance()->getRow('SELECT * FROM ps_firstdata_order WHERE id_cart = "' . $cartID . '"');
   
        
        $total = number_format($totAmount, 2);
        $txn_type = $result['payment_type'];
        $amt = number_format($refundAmt, 2);
        $this->logger->logInfo("txn_type: $txn_type, amount: $amt, total: $total");

        $method = "";        
        if ($txn_type == "preauth" && $amt == $total) {                       
            $method = "void";            
        } else if ($txn_type == "sale") {          
            $method = "return";            
        } else {             
            $message = $this->l('Partial amount refund is not possible on preauth transaction');
            $this->addNewPrivateMessage($order_id, $message);
            return false;
        }
          
        if ($method != "") {
            $reason = '';
            $request_param = array();
            $request_param['order_id'] = $result['id_order'];
            $request_param['method'] = $method;
            $request_param['order_hash_id'] = $result['id_order'];
            $request_param['transaction_id'] = $result['id_transaction'];
            $request_param['order_currency'] = $result['currency_code'];
            $request_param['amount'] = $amt;
            $request_param['reason'] = $reason;
            $this->logger->logInfo(print_r($request_param, true));
             
            $result = FiservApiHandler::refund_transaction($request_param);
           
            $history = new OrderHistory();
            $history->id_order = $order_id;

            $this->logger->logInfo($result);
      
            if (isset($result->ipgapi_ApprovalCode) && isset($result->ipgapi_TransactionResult)) {
                $approvalcode = substr($result->ipgapi_ApprovalCode, 0, 1);
                if ($approvalcode == 'Y' && strtolower($result->method) == "return") {                                      
                    $message = 'Refunded ['.$amt.'] on ' . date("Y-m-d h:i:s") . ' with Transaction ID = ' . $result->ipgapi_IpgTransactionId . ' using ' . strtoupper($result->method);                   
                    $this->addNewPrivateMessage((int) $order_id, $message); 
                    $history->changeIdOrderState(((int)(Configuration::get('PS_OS_REFUND'))), $order_id);
                }

                if ($approvalcode == 'Y' && strtolower($result->method) == "void") {
                    $message = 'Refunded ['.$amt.'] on ' . date("Y-m-d h:i:s") . ' with Transaction ID = ' . $result->ipgapi_IpgTransactionId . ' using ' . strtoupper($result->method);                   
                    $this->addNewPrivateMessage((int) $order_id, $message);  
                    $history->changeIdOrderState(((int)(Configuration::get('PS_OS_REFUND'))), $order_id);

                }

                if ($approvalcode == 'N') {                    
                    $message = 'Refund Failed: ' . $response->ipgapi_ErrorMessage;
                    $this->addNewPrivateMessage((int) $order_id, $message);

                }                                
               
                return isset($result->ipgapi_ErrorMessage) ? Tools::displayError($result->ipgapi_ErrorMessage) : false;
            }
        }  

    }


    public function addNewPrivateMessage($id_order, $message) {
        if (!(bool) $id_order) {
            return false;
        }
        $new_message = new Message();
        $message = strip_tags($message, '<br>');

        $new_message->message = $message;
        $new_message->id_order = (int) $id_order;
        $new_message->private = 1;
        return $new_message->add();
    }
    
    public static function getAvailableOptions($cart = null) {
        // multi payment options
        $options = @unserialize(Configuration::get('INS_MULTI_OPTIONS'));
     

        if (!is_array($options) || empty($options)) {
            return array();
        }

        if (!$cart) {
            return $options; // all options
        }

        $enabled_options = array();
        foreach ($options as $key => $option) {
            $min = $option['min_amount'];
            $max = $option['max_amount'];

            if ((empty($min) || $cart->getOrderTotal() >= $min) && (empty($max) || $cart->getOrderTotal() <= $max)) {
                $default = is_string($option['label']) ? $option['label'] : $option['count'] . ' x';
                $option_label = is_array($option['label']) && isset($option['label'][$cart->id_lang]) ?
                        $option['label'][$cart->id_lang] : $default;

                $option['localized_label'] = $option_label;

                $enabled_options[$key] = $option;
            }
        }
      
        return $enabled_options;
    }

    public function getTplVars($cart) {
        $vars['ins_multi_options'] = self::getAvailableOptions($cart);
        return $vars;
    }

    /**
     * Local Payment options supported in IPG configuration
     */
    public function paymentOptionsValues() {
        return array(
            "M" => "MasterCard",
            "V" => "Visa (Credit/Debit/Electron/Delta)",
            "A" => "American Express",
            "C" => "Diners",
            "J" => "JCB",
            "debitDE" => "SEPA Direct Debit",
            "CA" => "Cabal",
            "giropay" => "Giropay",
            "ideal" => "iDEAL",
            "klarna" => "Klarna",
            "indiawallet" => "Local Wallets India",
            "emi" => "Equated Monthly Instalments (EMI)",
            "MA" => "Maestro",
            "maestroUK" => "Maestro UK",
            "masterpass" => "MasterPass",
            "netbanking" => "Netbanking (India)",
            "paypal" => "PayPal",
            "RU" => "RuPay",
            "sofort" => "SOFORT Banking (Überweisung)	",
            "SO" => "Sorocred",
            "BCMC" => "Bancontact",
            "EL" =>  "ELO",
            "hipercard" => "HIPER/HIPERCARD",
            "mexicoLocal" => "MEXICOLOCAL",
        );
    }

    public function paymentOptions() {
        return array(
            "M" => array("card_support" => true, "shipping_support" => true),
            "V" => array("card_support" => true, "shipping_support" => true),
            "A" => array("card_support" => true, "shipping_support" => true),
            "C" => array("card_support" => true, "shipping_support" => true),
            "J" => array("card_support" => true, "shipping_support" => true),
            "debitDE" => array("card_support" => false, "shipping_support" => true),
            "CA" => array("card_support" => true, "shipping_support" => true),
            "giropay" => array("card_support" => false, "shipping_support" => true),
            "ideal" => array("card_support" => false, "shipping_support" => true),
            "klarna" => array("card_support" => false, "shipping_support" => true),
            "indiawallet" => array("card_support" => false, "shipping_support" => false),
            "emi" => array("card_support" => true, "shipping_support" => true),
            "MA" => array("card_support" => true, "shipping_support" => true),
            "maestroUK" => array("card_support" => true, "shipping_support" => true),
            "masterpass" => array("card_support" => false, "shipping_support" => true),
            "netbanking" => array("card_support" => false, "shipping_support" => false),
            "paypal" => array("card_support" => false, "shipping_support" => true),
            "RU" => array("card_support" => true, "shipping_support" => true),
            "sofort" => array("card_support" => false, "shipping_support" => true),
            "SO" => array("card_support" => true, "shipping_support" => true),
            "BCMC" => array("card_support" => true, "shipping_support" => true),
            "EL" => array("card_support" => true),
            "hipercard" => array("card_support" => true),
            "mexicoLocal" => array( "card_support" => true),
            );
    }

    // Validate response hash
    function validateGatewayHash($responsedata, $isNotify) {
        if (empty($responsedata) || !isset($responsedata['oid'])) {
            FiservTools::getLogger()->logInfo('No valid transaction data was passed to validate the order!');            
            return false;
        }
        $return = true;
        $env = Configuration::get('ENV');
        if ($env == 'TEST') {
            $storename = Configuration::get('TEST_STORE_NAME');
            $sharedsecret = Configuration::get('TEST_SHARED_SECERT');
        } else {
            $storename = Configuration::get('PROD_STORE_NAME');
            $sharedsecret = Configuration::get('PROD_SHARED_SECERT');
        }
        $chargetotal = $responsedata['chargetotal'];
        $txndatetime = $responsedata['txndatetime'];
        $approvalcode = $responsedata['approval_code'];
        $currencyCode = $responsedata['currency'];

        if ($isNotify == true) {
            FiservTools::getLogger()->logInfo('Hash Notify for Order Id: ' . $responsedata['oid'] . ' - Info');
            $hashValue = sha1(bin2hex($chargetotal . $sharedsecret . $currencyCode . $txndatetime . $storename . $approvalcode));

            if ($hashValue != $responsedata['notification_hash']) {
                FiservTools::getLogger()->logInfo('Invalid Notification Hash received from gateway Order Id: ' . $responsedata['oid'] . ' - Error');
                $return = false;
            }
        } else {
            FiservTools::getLogger()->logInfo('Hash Response for Order Id: ' . $responsedata['oid'] . ' - Info');
            $hashValue = sha1(bin2hex($sharedsecret . $approvalcode . $chargetotal . $currencyCode . $txndatetime . $storename));
            if ($hashValue != $responsedata['response_hash']) {
                FiservTools::getLogger()->logInfo('Invalid Response Hash received from gateway Order Id: ' . $responsedata['oid'] . ' - Error');
                $return = false;
            }
        }
        return $return;
    }

    /**
     * add translation system
     */

    public function isUsingNewTranslationSystem()
    {
        return true;
    }
}
