<?php

if (!defined('_PS_VERSION_')) {
    exit;
}

class FiservTools {
    /* fields lists */

    public static $multi_lang_fields = array();
    public static $amount_fields = array();
    public static $group_amount_fields = array();

    public static function checkCreditCard($inp) {
        /* @var Fiserv */
        $Fiserv = Module::getInstanceByName('Fiserv');

        $invalid_msg = $Fiserv->l('The field %1$s of your %2$s is invalid.', 'Fiservtools');
        $empty_msg = $Fiserv->l('The field %1$s of your %2$s is mandatory.', 'Fiservtools');

        $cardNumber = $inp['card_number'];
        $expiryMonth = $inp['expiry_month'];
        $expiryYear = $inp['expiry_year'];
        $cvv = $inp['cvv'];
        $expiryCard = $expiryMonth . '/' . $expiryYear;

        $errors = array();
        if (empty($cardNumber)) {
            $errors[] = sprintf($empty_msg, $Fiserv->l('Card Number', 'Fiservtools'), $cardNumber);
        } elseif (!preg_match("/^([0-9]{14,19})$/", $cardNumber)) {
            $errors[] = sprintf($invalid_msg, $Fiserv->l('Card Number', 'Fiservtools'), $cardNumber);
        }

        if (empty($cvv)) {
            $errors[] = sprintf($empty_msg, $Fiserv->l('CVV Number', 'Fiservtools'), $cvv);
        } elseif (!preg_match("/^([0-9]{2,4})$/", $cvv)) {
            $errors[] = sprintf($invalid_msg, $Fiserv->l('CVV Number', 'Fiservtools'), $cvv);
        }

        $curdate = date('Y-m-d');
        if (empty($expiryMonth)) {
            $errors[] = sprintf($empty_msg, $Fiserv->l('Expiry Month', 'Fiservtools'), $expiryMonth);
        } elseif (empty($expiryYear)) {
            $errors[] = sprintf($empty_msg, $Fiserv->l('Expiry Year', 'Fiservtools'), $expiryYear);
        } elseif ($expiryMonth < date("m") && $expiryYear < date("Y")) {
            $errors[] = sprintf($invalid_msg, $Fiserv->l('Expiry Month & Year', 'Fiservtools'), $expiryCard);
        }

        return $errors;
    }

    public static function checkLocalPayment($pm) {
        /* @var Fiserv */
        $Fiserv = Module::getInstanceByName('Fiserv');

        $empty_msg = $Fiserv->l('The field %1$s of your %2$s is mandatory.', 'Fiservtools');

        $errors = array();
        if (empty($pm)) {
            $errors[] = sprintf($empty_msg, $Fiserv->l('Local Methods', 'Fiservtools'), $pm);
        }
        return $errors;
    }

    /**
     * Return the list of configuration parameters with their Fiserv names and default values.
     *
     * @return array
     */
    public static function getAdminParameters() {
        // NB : keys are 32 chars max
        $params = array(
            array('key' => 'LOGS', 'default' => 'no', 'label' => 'Logs'),
            array('key' => 'COUNTRY', 'name' => '', 'default' => '', 'label' => 'COUNTRY'),
            array('key' => 'RESELLER', 'default' => '', 'label' => 'RESELLER'),
            array('key' => 'PLUGIN_NAME', 'name' => '', 'default' => '', 'label' => 'Plugin Name'),
            array('key' => 'PLUGIN_DESC', 'name' => '', 'default' => '', 'label' => 'Plugin Description'),
            array('key' => 'PLUGIN_LOGO', 'default' => '', 'label' => 'Logo'),
            array('key' => 'LOGO_DISPLAY', 'default' => 'no', 'label' => 'Logo Display'),
            array('key' => 'CUSTOMER_DETAIL_TITLE', 'name' => '', 'default' => '', 'label' => 'Customer Detail Title'),
            array('key' => 'CUSTOMER_DETAIL', 'name' => '', 'default' => '', 'label' => 'Customer Detail'),
            array('key' => 'CONTACT_SUPPORT_TITLE', 'name' => '', 'default' => '', 'label' => 'Contact Support Title'),
            array('key' => 'CONTACT_SUPPORT', 'name' => '', 'default' => '', 'label' => 'Contact Support'),
            array('key' => 'ENV', 'default' => 'TEST', 'label' => 'Environment'),
            array('key' => 'TEST_SHARED_SECERT', 'name' => '', 'default' => '', 'label' => 'Shared Secret'),
            array('key' => 'TEST_STORE_NAME', 'name' => '', 'default' => '', 'label' => 'Store Name'),
            array('key' => 'TEST_SUCCESS_URL', 'name' => '', 'default' => '', 'label' => 'Response Success URL'),
            array('key' => 'TEST_FAILURE_URL', 'name' => '', 'default' => '', 'label' => 'Response Failure URL'),
            array('key' => 'TEST_NOTIFICATION_URL', 'name' => '', 'default' => '', 'label' => 'Notification URL'),
            array('key' => 'PROD_SHARED_SECERT', 'name' => '', 'default' => '', 'label' => 'Shared Secret'),
            array('key' => 'PROD_STORE_NAME', 'name' => '', 'default' => '', 'label' => 'Store Name'),
            array('key' => 'PROD_SUCCESS_URL', 'name' => '', 'default' => '', 'label' => 'Response Success URL'),
            array('key' => 'PROD_FAILURE_URL', 'name' => '', 'default' => '', 'label' => 'Response Failure URL'),
            array('key' => 'PROD_NOTIFICATION_URL', 'name' => '', 'default' => '', 'label' => 'Notification URL'),
            array('key' => 'AUTHORISATION', 'default' => 'yes', 'label' => 'Authorization'),
            array('key' => 'CHECKOUT_MODE', 'name' => '', 'default' => 'classic', 'label' => 'Checkout Mode'),
            array('key' => 'PAYMENT_MODE', 'name' => '', 'default' => 'payonly', 'label' => 'Payment Mode'),
            array('key' => 'TOKENISATION', 'default' => 'no', 'label' => 'Tokenisation'),
            array('key' => 'DYNAMIN_DESC', 'name' => '', 'default' => '', 'label' => 'Dynamic Descriptor'),
            array('key' => 'SECURE_PAY', 'name' => '', 'default' => 'active', 'label' => '3D Secure'),
            array('key' => 'DCC_SKIP_OFFER', 'name' => '', 'default' => 'no', 'label' => 'Dynamic Currency Conversion'),
            array('key' => 'AUTH_METHOD', 'default' => 'hosted', 'label' => 'Authorisation Method'),
            array('key' => 'INSTALMENT_ENABLED', 'default' => 'no', 'label' => 'Activation'),
            array('key' => 'INS_INT_MEXICO', 'default' => 'no', 'label' => 'Installment Interest Mexico'),
            array('key' => 'LOCAL_PAYMENT', 'default' => 'no', 'label' => 'Local Payment'),
            array('key' => 'PAYMENT_METHOD', 'default' => '', 'label' => 'Local Payment Method'),
            array('key' => 'INS_MULTI_OPTIONS', 'default' => array(), 'label' => 'Payment in several times - Payment options'),
            array('key' => 'IPG_API_USERNAME', 'name' => '', 'default' => '', 'label' => 'Api Username'),
            array('key' => 'IPG_API_PASSWORD', 'name' => '', 'default' => '', 'label' => 'Api Password'),
            array('key' => 'IPG_API_CERTIFICATE_PASSWORD', 'name' => '', 'default' => '', 'label' => 'Api Certificate Password'),
            array('key' => 'IPG_API_SERVER_TRUST_PEM', 'name' => '', 'default' => array(), 'label' => 'Api Server Trust PEM'),
            array('key' => 'IPG_API_CERTIFICATE_PEM', 'name' => '', 'default' => array(), 'label' => 'API Certificate PEM'),
            array('key' => 'IPG_API_CERTIFICATE_KEY', 'name' => '', 'default' => array(), 'label' => 'API Certificate Key'),
            array('key' => 'IPG_API_URL', 'name' => '', 'default' => '', 'label' => 'API URL'),
        );

        return $params;
    }

    private static $logger;

    public static function getLogger() {
        if (!self::$logger) {
            self::$logger = new FiservFileLogger(Configuration::get('LOGS') != 'no');

            $logs_dir = _PS_ROOT_DIR_ .DIRECTORY_SEPARATOR.'var'.DIRECTORY_SEPARATOR.'logs'.DIRECTORY_SEPARATOR;
            if (!file_exists($logs_dir)) {
                $logs_dir = _PS_ROOT_DIR_ . '/log/';
            }
            self::$logger->setFilename($logs_dir . 'Fiserv.log');
        }

        return self::$logger;
    }

    public static function getTemplatePath($tpl) {
        if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            return 'module:Fiserv/views/templates/front/' . $tpl;
        }

        return $tpl;
    }

    public function addOrderNoteForEMI($id_order, $message) {
        if (!(bool) $id_order) {
            return false;
        }
        $new_message = new Message();
        $message = strip_tags($message, '<br>');

        $new_message->message = $message;
        $new_message->id_order = (int) $id_order;
        $new_message->private = 1;
        return $new_message->add();
    }


    public static function sanitizeRequest($req){
      
            if (is_array($req)){
                $req['cardnumber']="";
                $req['expmonth']="";
                $req['expyear']="";
                $req['cvm']="";
            }
         

            return $req;
    }
}   